package EncodingDetect;
use warnings;
use strict;
use base 'Exporter';

our @EXPORT = qw( find_html_encoding );

sub _prescan_get_attribute {
    my ($string) = @_;
    if ($string =~ m{[\s/]*([^=\s/>]+)\s*=\s*
                     (?:"([^"]+)"|'([^']+)'|([^\s>])+)}x) {
        my ($name, $value) = ($1, $+);
        return ($+[0], lc $name, lc $value);
    } else {
        return (0, undef, undef);
    }
}

sub find_html_encoding {
    my ($prefix) = @_;
    my $encoding = undef;
    if (substr($prefix, 0, 2) eq "\xFE\xFF") {
        $encoding = 'UTF-16BE';
    } elsif (substr($prefix, 0, 2) eq "\xFF\xFE") {
        $encoding = 'UTF-16LE';
    } elsif (substr($prefix, 0, 3) eq "\xEF\xBB\xBF") {
        $encoding = 'UTF-8';
    } else {
        my $pos = 0;
        while (not defined $encoding and $pos >= 0 && $pos < length $prefix) {
            if (substr($prefix, $pos, 4) eq '<!--') {
                $pos = index($prefix, $pos, '-->');
            } elsif (substr($prefix, $pos, 6) =~ m{<meta[\s/]}i) {
                $pos += 6;
                for (;;) {
                    my ($pos_offset, $name, $value) =
                        _prescan_get_attribute(substr($prefix, $pos));
                    $pos += $pos_offset;
                    last if not defined $name;
                    if ($name eq 'charset') {
                        $encoding = $value;
                        if ($encoding =~ /utf-?16/i) {
                            $encoding = 'UTF-8';
                        }
                    } elsif ($name eq 'content' and $value =~
                            /charset\s*=\s*(?:"([^"]+)"|'([^']+)'|([^\s]+))/) {
                        $encoding = $+;
                    }
                }
                $pos = index($prefix, '>', $pos);
            } elsif (substr($prefix, $pos) =~ m{^(?:</?[a-zA-Z]|<!|</|<?)[^>]+}) {
                $pos += $+[0];
            } else {
                ++$pos;
            }
        }
    }
    if (not defined $encoding or not defined Encode::find_encoding($encoding)) {
        $encoding = 'Windows-1252';
    }
    return $encoding;
}

1;
